#!/usr/bin/env sh
# Copyright (C) 2008 Red Hat, Inc.
# Written by Daniel Kopecek <dkopecek@redhat.com>, Michel Samia <msamia@redhat.com>
# Mon Apr  7 14:28:36 CEST 2008
. "${TESTS_DIRS}/bash_defs.sh"
# --- Error IDs ------- #
const ID_DIRWORLDW=1
const ID_FILEWORLDGROUPWX=2
const ID_FILEWORLDW=3
const ID_GROUPNOTFOUND=4
const ID_NOPKG=5
const ID_SUIDSCRIPT=6
const ID_USERNOTFOUND=7
const ID_WRONGSYMLINK=8

STATCMD="$(which stat)"

for dir in  ${EXCLUDE}; do
	FIND_OPTIONS="${FIND_OPTIONS} -not -path '${dir}/*'"
done

if [[ -z "${INCLUDE}" ]]; then
	INCLUDE="/"
fi

while read path; do
	STAT=(`${STATCMD} --printf='%U\t%G\t%a\t%F' "${path}"`)
	
	if [[ "${STAT[0]}" == "UNKNOWN" ]]; then
		report 'ERROR' $ID_USERNOTFOUND "User ID of ${STAT[3]} \"${path}\" doesn't exist in user database."
	fi
	
	if [[ "${STAT[1]}" == "UNKNOWN" ]]; then
		report 'ERROR' $ID_GROUPNOTFOUND "Group ID of ${STAT[3]} \"${path}\" doesn't exist in group database."
	fi

	if [[ "${STAT[3]}" == "regular" ]]; then	
		if [[ $((8#${STAT[2]} & 2)) == 2 ]]; then
		    report 'WARNING' $ID_FILEWORLDW "File \"${path}\" is world write-able."
		fi

		if [[ $((8#${STAT[2]} & 18)) > 0 && \
		      $((8#${STAT[2]} &  9)) > 0 ]]; then
		    report 'WARNING' $ID_FILEWORLDGROUPWX "File \"${path}\" is world/group write-able and world/group executable."
		fi
		
		if [[ $((8#${STAT[2]} & 73)) > 0 && \
		      $((8#${STAT[2]} & 3072)) > 0 && \
		      16#$(od -t x1 -N 2 -A n "${path}" | tr -d '[[:space:]]') == 16#2321 ]]; then
		    report 'ERROR' $ID_SUIDSCRIPT "File \"${path}\" is a SUID script."    
		fi
			
		if [[ ${RPMCHECK} == 1 && $((8#${STAT[2]} & 73)) > 0 ]]; then
		    rpmquery -f "${path}" > /dev/null
		    if (( $? != 0 )); then
		    	report 'ERROR' $ID_NOPKG "Can't find package that owns executable file \"${path}\"."
		    fi
		fi
	elif [[ "${STAT[3]}" == "symbolic" ]]; then
		${STATCMD} -L "${path}" &>/dev/null
		if (( $? != 0 )); then
			target="$(readlink "${path}")"
			report 'ERROR' $ID_WRONGSYMLINK "Symbolic link \"${path}\" points to file \"${target}\", which does not exist."
		fi
	fi

	if [[ $((8#${STAT[2]} & 7)) == 7   && \
	      "${STAT[3]}" == "directory" && \
	      $((8#${STAT[2]} & 512)) == 0 ]]; then
	    report 'WARNING' $ID_DIRWORLDW "Directory \"${path}\" is world write-able and the sticky bit is not set."
	fi


done <<EOF
`find ${INCLUDE} -fstype ext3 -or -fstype ext2 ${FIND_OPTIONS}`
EOF

test_exit ${E_OK}
